# -*- coding: utf8 -*-

from __future__ import unicode_literals

import datetime

parser_class = "sentifit270"


def test_encode_order_compliance_astm(self):
    message = self.order()
    expected = """
    H|\\^&|||Host|||||||||20130715141912
    P|1|036666666|||NYME ANO||20110513|F|||||LAGRANGE Joseph-Louis
    O|1|123456||^^^A\\^^^B|||20130513114242||||||||Serum|Serum
    C|1|L|Patient commentaire no1|G
    C|2|L|Patient commentaire no2|G
    C|3|L|Sample commentaire|G
    L|1
    """
    assert message == expected


def test_encode_order_non_compliance_astm(self):
    self.configure(sentifit_non_compliance_astm=True)
    message = self.order()
    expected = """
    H|\\^&|||Host|||||||||20130715141912
    P|1|036666666|||NYME ANO||20110513|F|||||LAGRANGE Joseph-Louis
    O|1|123456||A^B|||20130513114242||||||||Serum|Serum
    C|1|L|Patient commentaire no1|G
    C|2|L|Patient commentaire no2|G
    C|3|L|Sample commentaire|G
    L|1
    """
    assert message == expected


def test_query(self):
    query = self.decode(
        """
    H|\\^&|||Analyzer|||||||||20190702134002
    Q|1|999999SEL||||||||||N
    L|1"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "999999SEL", "type": "patient"}],
    }
    assert query == expected


def test_result_production(self):
    date = datetime.datetime(2019, 7, 2, 14, 32, 29)
    result = self.decode(
        """
    H|\\^&|||Analyzer|||||||||20190702143231
    P|1|20302|||TEST01^Patient^Pre||20010101|M|||||||0||||||||||^^^|
    C|1|I||G
    O|1|999999SEL||^^^01|||20190702000000||0|||||||02
    C|1|I||G
    R|1|^^^01|15.00^Negative|ng/mL||<||||||20190702143229
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "999999SEL",
                "type": "patient",
                "specimen": "02",
                "results": [
                    {
                        "code": "01",
                        "value": "15.00",
                        "unit": "ng/mL",
                        "flags": ["<"],
                        "comments": [],
                        "date": date,
                    },
                    {
                        "code": "01QUAL",
                        "value": "Negative",
                        "unit": "ng/mL",
                        "flags": ["<"],
                        "comments": [],
                        "date": date,
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_result(self):
    result = self.decode(
        """
    H|\\^&|||Analizer|||||||||20010111055300
    P|1|PID2734|||Patient Name||19630501|M|Race1||||Phys. Name|1234567890123|1
    O|1|001||^^^1|||||||||||01|01
    C|1|L|TestOrder1|G
    R|1|^^^1|15.265|mg/mL||||||||20010110121530
    O|2|001||^^^3|||||||||||01|01
    R|1|^^^3|18.052|mg/mL||||||||20010110121830
    P|2|PID2738||Patient Name||19870501|M|Race1||||Phys. Name|1231567890153|1
    O|1|890051||^^^5|||||||||||01|01
    C|1|L|TestOrder2|G
    R|1|^^^5|5.265|mg/mL||||||||20010110151530
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "1",
                        "comments": ["TestOrder1"],
                        "date": datetime.datetime(2001, 1, 10, 12, 15, 30),
                        "flags": [],
                        "unit": "mg/mL",
                        "value": "15.265",
                    },
                    {
                        "code": "3",
                        "comments": ["TestOrder1"],
                        "date": datetime.datetime(2001, 1, 10, 12, 18, 30),
                        "flags": [],
                        "unit": "mg/mL",
                        "value": "18.052",
                    },
                ],
                "sample_id": "001",
                "type": "patient",
                "specimen": "01",
            },
            {
                "sample_id": "890051",
                "type": "patient",
                "specimen": "01",
                "results": [
                    {
                        "code": "5",
                        "comments": ["TestOrder2"],
                        "date": datetime.datetime(2001, 1, 10, 15, 15, 30),
                        "flags": [],
                        "unit": "mg/mL",
                        "value": "5.265",
                    }
                ],
            },
        ],
    }
    assert result == expected


def test_order_batch(self):
    query = """H|\\^&|||Analyzer|||||||||20010111055300
    Q|1|ALL||||||||||N
    L|1
    """
    expected = """H|\\^&|||Host|||||||||20010111055303
    P|1|036666666|||NYME ANO||20110513|F|||||LAGRANGE Joseph-Louis
    O|1|123456||^^^A\\^^^B|||20130513114242||||||||Serum|Serum
    C|1|L|Patient commentaire no1|G
    C|2|L|Patient commentaire no2|G
    C|3|L|Sample commentaire|G
    L|1
    """
    result = self.order(
        context={"query": query, "now": datetime.datetime(2001, 1, 11, 5, 53, 3)}
    )
    assert result == expected


def test_order_real_time(self):
    query = """H|\\^&|||Analizer|||||||||20010111055300
    Q|1|9100000000001||||||||||N
    L|1
    """
    expected = """H|\\^&|||Host|||||||||20010111055303
    P|1|036666666|||NYME ANO||20110513|F|||||LAGRANGE Joseph-Louis
    O|1|123456||^^^A\\^^^B|||20130513114242||||||||Serum|Serum
    C|1|L|Patient commentaire no1|G
    C|2|L|Patient commentaire no2|G
    C|3|L|Sample commentaire|G
    L|1
    """
    result = self.order(
        context={"query": query, "now": datetime.datetime(2001, 1, 11, 5, 53, 3)}
    )
    assert result == expected


def test_QC(self):
    result = self.decode(
        """
    H|\\^&|||Analizer|||||||||20100929100355
    P|1|97000001||||||U||||||0
    C|1|L||G
    O|1|97000001||^^^02|||20100929000000||0||||||05|05
    C|L||G
    R|1|^^^02|-2045.96|||<||||||20100929100355
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "97000001",
                "control_lot": "97000001",
                "type": "control",
                "specimen": "05",
                "results": [
                    {
                        "code": "02",
                        "comments": [],
                        "date": datetime.datetime(2010, 9, 29, 10, 3, 55),
                        "flags": ["<"],
                        "unit": "",
                        "value": "-2045.96",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_control_trame_production(self):
    result = self.decode(
        """
    H|\\^&|||Analyzer|||||||||20190702153307
    P|1|97000001||||||U|||||||0||||||||||^^^|
    C|1|I||G
    O|1|97000001||01|||||0|||||||02
    C|1|I||G
    R|1|01|90.20|||00^00^00^00^00^00^00^00^00^00^00^00^00^00||||||20190702153305
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "97000001",
                "control_lot": "97000001",
                "type": "control",
                "specimen": "02",
                "results": [
                    {
                        "code": "01",
                        "value": "90.20",
                        "flags": ["00"],
                        "unit": "",
                        "comments": [],
                        "date": datetime.datetime(2019, 7, 2, 15, 33, 5),
                    }
                ],
            }
        ],
    }
    assert result == expected
